@@ -24,9 +24,12 @@ pub mod group_theory_preset;
 pub mod longest_common_prefix;
 pub mod longest_increasing_sequence;
 pub mod matrix;
+pub mod matrix_constant;
 pub mod matrix_dynamic;
-pub mod modular;
-pub mod modular_dynamic;
+pub mod matrix_runtime_static;
+pub mod matrix_static;
+pub mod modular_constant;
+pub mod modular_runtime_static;
 pub mod modular_static;
 pub mod power;
 pub mod primality;
