@@ -1,18 +1,21 @@
 use crate::group_theory;
 
-pub trait Power<S = Self, T = group_theory::Multiplicative>: group_theory::Monoid<S, T> {
-    fn pow(value: &S, expontent: usize) -> S;
+pub trait Power<T: group_theory::BinaryOperationIdentifier>: group_theory::Monoid<T> {
+    fn pow(value: &Self, expontent: usize) -> Self;
 }
 
-impl<S, T, M: group_theory::Monoid<S, T>> Power<S, T> for M {
+impl<S: group_theory::Monoid<T>, T> Power<T> for S
+where
+    T: crate::group_theory::BinaryOperationIdentifier,
+{
     fn pow(value: &S, exponent: usize) -> S {
         if exponent == 0 {
-            return M::identity();
+            return S::identity();
         }
         let mut y = Self::pow(value, exponent >> 1);
-        y = M::operate(&y, &y);
+        y = S::operate(&y, &y);
         if exponent & 1 == 1 {
-            y = M::operate(&y, &value);
+            y = S::operate(&y, &value);
         }
         y
     }
