@@ -1,29 +1,27 @@
-use crate::{
-    bitwise,
-    group_theory::{Additive, Monoid},
-};
+use crate::{bitwise, group_theory::Monoid};
 
 /// Node Indices (case $4 \lt |given array| \le 8$)
 /// |1                      |2
 /// |2          |3          |4
 /// |4    |5    |6    |7    |8
 /// |8 |9 |10|11|12|13|14|15|16
-pub struct SegmentTree<M: Monoid<S, T>, S = M, T = Additive> {
+pub struct SegmentTree<S: Monoid<T>, T: crate::group_theory::BinaryOperationIdentifier> {
     phantom_t: std::marker::PhantomData<T>,
-    phantom_m: std::marker::PhantomData<M>,
     size: usize,
     data: Vec<S>,
 }
 
-impl<M: Monoid<S, T>, S: Clone, T> From<&[S]> for SegmentTree<M, S, T> {
+impl<S: Monoid<T> + Clone, T> From<&[S]> for SegmentTree<S, T>
+where
+    T: crate::group_theory::BinaryOperationIdentifier,
+{
     fn from(slice: &[S]) -> Self {
         let size = slice.len();
         let n = size.next_power_of_two();
-        let mut data = vec![M::identity(); n << 1];
+        let mut data = vec![S::identity(); n << 1];
         data[n..(n + size)].clone_from_slice(slice);
         let mut seg = Self {
             phantom_t: std::marker::PhantomData,
-            phantom_m: std::marker::PhantomData,
             size: slice.len(),
             data,
         };
@@ -34,19 +32,22 @@ impl<M: Monoid<S, T>, S: Clone, T> From<&[S]> for SegmentTree<M, S, T> {
     }
 }
 
-impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
+impl<S: Monoid<T>, T> SegmentTree<S, T>
+where
+    T: crate::group_theory::BinaryOperationIdentifier,
+{
     pub fn new(size: usize) -> Self
     where
         S: Clone,
     {
-        (&vec![M::identity(); size]).as_slice().into()
+        (&vec![S::identity(); size]).as_slice().into()
     }
 
     pub fn size(&self) -> usize { self.size }
 
     fn merge_childs(&mut self, node_index: usize) {
         self.data[node_index] =
-            M::operate(&self.data[node_index << 1], &self.data[node_index << 1 | 1]);
+            S::operate(&self.data[node_index << 1], &self.data[node_index << 1 | 1]);
     }
 
     pub fn set_point(&mut self, array_index: usize, x: S) {
@@ -64,21 +65,21 @@ impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
         let n = self.data.len() >> 1;
         let mut left_node_index = n + left;
         let mut right_node_index = n + right;
-        let mut value_left = M::identity();
-        let mut value_right = M::identity();
+        let mut value_left = S::identity();
+        let mut value_right = S::identity();
         while left_node_index < right_node_index {
             if left_node_index & 1 == 1 {
-                value_left = M::operate(&value_left, &self.data[left_node_index]);
+                value_left = S::operate(&value_left, &self.data[left_node_index]);
                 left_node_index += 1;
             }
             if right_node_index & 1 == 1 {
                 right_node_index -= 1;
-                value_right = M::operate(&self.data[right_node_index], &value_right);
+                value_right = S::operate(&self.data[right_node_index], &value_right);
             }
             left_node_index >>= 1;
             right_node_index >>= 1;
         }
-        M::operate(&value_left, &value_right)
+        S::operate(&value_left, &value_right)
     }
 
     pub fn find_max_right<F>(&self, is_ok: &F, left: usize) -> usize
@@ -90,15 +91,15 @@ impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
             return self.size;
         }
         let n = self.data.len() >> 1;
-        let mut value = M::identity();
+        let mut value = S::identity();
         let mut node_index = n + left;
         loop {
             node_index = bitwise::shift_right_until_odd(node_index).unwrap(); // up to ceil
-            if !is_ok(&M::operate(&value, &self.data[node_index])) {
+            if !is_ok(&S::operate(&value, &self.data[node_index])) {
                 break;
             }
             // up one stair from left
-            value = M::operate(&value, &self.data[node_index]);
+            value = S::operate(&value, &self.data[node_index]);
             node_index += 1;
             if bitwise::lsb_number(node_index) == node_index {
                 // wall.
@@ -108,10 +109,10 @@ impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
         // down stairs to right
         while node_index < n {
             node_index <<= 1;
-            if !is_ok(&M::operate(&value, &self.data[node_index])) {
+            if !is_ok(&S::operate(&value, &self.data[node_index])) {
                 continue;
             }
-            value = M::operate(&value, &self.data[node_index]);
+            value = S::operate(&value, &self.data[node_index]);
             node_index += 1;
         }
         node_index - n
@@ -126,34 +127,37 @@ impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
             return 0;
         }
         let n = self.data.len() >> 1;
-        let mut value = M::identity();
+        let mut value = S::identity();
         let mut node_index = n + right;
         loop {
             assert!(node_index >= 1);
             node_index = bitwise::shift_right_until_odd(node_index).unwrap();
             assert!(node_index >= 1);
-            if !is_ok(&M::operate(&self.data[node_index - 1], &value)) {
+            if !is_ok(&S::operate(&self.data[node_index - 1], &value)) {
                 break;
             }
             node_index -= 1;
-            value = M::operate(&self.data[node_index as usize], &value);
+            value = S::operate(&self.data[node_index as usize], &value);
             if bitwise::lsb_number(node_index) == node_index {
                 return 0;
             }
         }
         while node_index < n {
             node_index <<= 1;
-            if !is_ok(&M::operate(&self.data[node_index - 1], &value)) {
+            if !is_ok(&S::operate(&self.data[node_index - 1], &value)) {
                 continue;
             }
             node_index -= 1;
-            value = M::operate(&self.data[node_index], &value);
+            value = S::operate(&self.data[node_index], &value);
         }
         node_index - n
     }
 }
 
-impl<M: Monoid<S, T>, S, T> std::ops::Index<usize> for SegmentTree<M, S, T> {
+impl<S: Monoid<T>, T> std::ops::Index<usize> for SegmentTree<S, T>
+where
+    T: crate::group_theory::BinaryOperationIdentifier,
+{
     type Output = S;
 
     fn index(&self, i: usize) -> &Self::Output {
@@ -163,7 +167,10 @@ impl<M: Monoid<S, T>, S, T> std::ops::Index<usize> for SegmentTree<M, S, T> {
 }
 
 /// Recursive Implementations for bench mark.
-impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
+impl<S: Monoid<T>, T> SegmentTree<S, T>
+where
+    T: crate::group_theory::BinaryOperationIdentifier,
+{
     pub fn get_range_recurse(&self, left: usize, right: usize) -> S {
         assert!(left <= right && right <= self.size);
         self._get_recurse(left, right, 0, self.data.len() >> 1, 1)
@@ -178,13 +185,13 @@ impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
         node_index: usize,
     ) -> S {
         if current_right <= left || right <= current_left {
-            return M::identity();
+            return S::identity();
         }
         if left <= current_left && current_right <= right {
-            return M::operate(&M::identity(), &self.data[node_index]);
+            return S::operate(&S::identity(), &self.data[node_index]);
         }
         let center = (current_left + current_right) >> 1;
-        M::operate(
+        S::operate(
             &self._get_recurse(left, right, current_left, center, node_index << 1),
             &self._get_recurse(left, right, center, current_right, node_index << 1 | 1),
         )
@@ -195,7 +202,7 @@ impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
         F: Fn(&S) -> bool,
     {
         assert!(left <= self.size);
-        self._max_right_recurse(is_ok, left, 0, self.data.len() >> 1, &mut M::identity(), 1)
+        self._max_right_recurse(is_ok, left, 0, self.data.len() >> 1, &mut S::identity(), 1)
     }
 
     /// find max right (current_left < right <= current_right)
@@ -221,9 +228,9 @@ impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
         }
         if left <= current_left
             && current_right <= self.size
-            && is_ok(&M::operate(current_value, &self.data[node_index]))
+            && is_ok(&S::operate(current_value, &self.data[node_index]))
         {
-            *current_value = M::operate(current_value, &self.data[node_index]);
+            *current_value = S::operate(current_value, &self.data[node_index]);
             return current_right;
         }
         if current_right - current_left == 1 {
@@ -256,7 +263,7 @@ impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
         F: Fn(&S) -> bool,
     {
         assert!(right <= self.size);
-        self._min_left_recurse(is_ok, right, 0, self.data.len() >> 1, &mut M::identity(), 1)
+        self._min_left_recurse(is_ok, right, 0, self.data.len() >> 1, &mut S::identity(), 1)
     }
 
     fn _min_left_recurse<F>(
@@ -274,8 +281,8 @@ impl<M: Monoid<S, T>, S, T> SegmentTree<M, S, T> {
         if current_left >= right {
             return right;
         }
-        if current_right <= right && is_ok(&M::operate(&self.data[node_index], current_value)) {
-            *current_value = M::operate(&self.data[node_index], current_value);
+        if current_right <= right && is_ok(&S::operate(&self.data[node_index], current_value)) {
+            *current_value = S::operate(&self.data[node_index], current_value);
             return current_left;
         }
         if current_right - current_left == 1 {
@@ -309,15 +316,18 @@ mod tests {
     use crate::group_theory::{Associative, BinaryOperation, Identity};
     #[test]
     fn test_as_monoid() {
-        impl BinaryOperation for usize {
+        struct Add;
+        impl crate::group_theory::BinaryOperationIdentifier for Add {}
+
+        impl BinaryOperation<Add> for usize {
             fn operate(x: &Self, y: &Self) -> Self { x + y }
         }
-        impl Associative for usize {}
-        impl Identity for usize {
+        impl Associative<Add> for usize {}
+        impl Identity<Add> for usize {
             fn identity() -> Self { 0 }
         }
 
-        let mut seg = super::SegmentTree::<usize>::new(10);
+        let mut seg = super::SegmentTree::<usize, Add>::new(10);
         assert_eq!(seg.get_range(0, 10), 0);
         assert_eq!(seg.get_range_recurse(0, 10), 0);
         seg.set_point(5, 5);
@@ -340,23 +350,23 @@ mod tests {
         assert_eq!(seg.find_min_left(is_ok, 6), 6);
         assert_eq!(seg.find_min_left_recurse(is_ok, 6), 6);
 
-        seg = super::SegmentTree::<usize>::new(0);
+        seg = super::SegmentTree::<usize, Add>::new(0);
         assert_eq!(seg.get_range(0, 0), 0);
     }
 
     #[test]
     fn test_wrapping_monoid() {
-        struct UsizeAdd;
-
-        impl BinaryOperation<usize> for UsizeAdd {
+        struct Add;
+        impl crate::group_theory::BinaryOperationIdentifier for Add {}
+        impl BinaryOperation<Add> for usize {
             fn operate(x: &usize, y: &usize) -> usize { x + y }
         }
-        impl Associative<usize> for UsizeAdd {}
-        impl Identity<usize> for UsizeAdd {
+        impl Associative<Add> for usize {}
+        impl Identity<Add> for usize {
             fn identity() -> usize { 0 }
         }
 
-        let mut seg = super::SegmentTree::<UsizeAdd, usize>::new(10);
+        let mut seg = super::SegmentTree::<usize, Add>::new(10);
         assert_eq!(seg.get_range(0, 10), 0);
         seg.set_point(0, 5);
         assert_eq!(seg.get_range(0, 10), 5);
